﻿/*
Copyright (c) 2002-2007, The Npgsql Development Team

Permission to use, copy, modify, and distribute this software and its documentation for any purpose, without fee, and without a written agreement is hereby granted, provided that the above copyright notice and this paragraph and the following two paragraphs appear in all copies.

IN NO EVENT SHALL THE NPGSQL DEVELOPMENT TEAM BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE NPGSQL DEVELOPMENT TEAM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

THE NPGSQL DEVELOPMENT TEAM SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE NPGSQL DEVELOPMENT TEAM HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

 * 
 */


using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;
using Npgsql;
using System.Data;
namespace dhtmlxConnectors
{
    /// <summary>
    /// PostgreSQL compatible implementation of idhtmlxDatabaseAdapter interface
    /// adapter uses Npgsql .Net data provider for PostgreSQL, which can be downloaded from
    /// http://pgfoundry.org   
    /// </summary>
    public class PostgreSQLAdapter : MSSQLAdapter, IdhtmlxDatabaseAdapter
    {

        /// <summary>
        /// Creates initialized opened IDbConnection object
        /// </summary>
        /// <param name="ConnectionString">ConnectionString to initialize connection by</param>
        /// <returns></returns>
        protected override IDbConnection CreateConnection(string ConnectionString)
        {
            return new NpgsqlConnection(ConnectionString);
        }

        /// <summary>
        /// Creates IDbCommand object
        /// </summary>
        /// <param name="Query">Query to initialize command with</param>
        /// <param name="Connection">Connection to initialize command with</param>
        /// <returns>IDbCommand object</returns>
        protected override IDbCommand CreateCommand(string Query, IDbConnection Connection)
        {
            return new NpgsqlCommand(Query, Connection as NpgsqlConnection);
        }

        /// <summary>
        /// Creates IDbDataAdapter
        /// </summary>
        /// <param name="Command">Command to initialize adapter with</param>
        /// <returns>IDbDataAdapter object</returns>
        protected override IDbDataAdapter CreateDataAdapter(IDbCommand Command)
        {
            return new NpgsqlDataAdapter(Command as NpgsqlCommand);
        }

        /// <summary>
        /// Creates SQL Insert query
        /// </summary>
        /// <param name="TableName">Table name to insert record into</param>
        /// <param name="ColumnValues">Values to insert</param>
        /// <returns>SQL Insert query</returns>
        protected override string CreateInsertQuery(string TableName, Dictionary<Field, string> ColumnValues)
        {
            #region LOG ENTRY
#if !NO_LOG
            Log.WriteLine(this, string.Format("Creating insert query from TableName: {0}, Fields-Values: {1}", TableName, Tools.Join(ColumnValues.Select(item => item.Key.ToString() + " = " + item.Value), ", ")));
#endif
            #endregion
            string insertQuery = string.Format("INSERT INTO {0} ({{0}}) VALUES ({{1}});", TableName);
            List<string> insertColumns = new List<string>();
            List<string> insertValues = new List<string>();

            foreach (Field insertField in ColumnValues.Keys)
            {
                insertColumns.Add(insertField.InternalName);
                insertValues.Add("'" + Tools.EscapeQueryValue(ColumnValues[insertField]) + "'");
            }
            string result = string.Format(insertQuery, string.Join(", ", insertColumns.ToArray()), string.Join(", ", insertValues.ToArray()));
            #region LOG ENTRY
#if !NO_LOG
            Log.WriteLine(this, "Insert query: " + result);
#endif
            #endregion
            return result;
        }

        /// <summary>
        /// Creates query to retrieve last inserted id
        /// </summary>
        /// <param name="TableName">Table name</param>      
        /// <param name="primary">Table primary key</param>
        /// <returns>SQL Select query</returns>
        protected string CreateLastInsertedQuery(string TableName, Field primary)
        {
            string query = string.Format("SELECT {0} FROM {1} ORDER BY {2} DESC LIMIT 1 OFFSET 0 ;", primary.InternalName, TableName, primary.InternalName);// SELECT SCOPE_IDENTITY();
            return query;
        }

        /// <summary>
        /// Executes insert query
        /// </summary>
        /// <param name="TableName">Table name to insert record in</param>
        /// <param name="ColumnValues">Column values to insert</param>
        /// <param name="PrimaryKeyField">Primary Key column</param>
        /// <param name="PrimaryKeyValue">Primary key column value</param>
        /// <param name="AllFields">Collection of all tables fields</param>
        /// <returns>ID of inserted record</returns>
        public override object ExecuteInsertQuery(string TableName, Dictionary<Field, string> ColumnValues, Field PrimaryKeyField, object PrimaryKeyValue, dhtmlxFieldsCollection AllFields)
        {
            #region LOG ENTRY
#if !NO_LOG
            Log.WriteLine(this, "Entering: ExecuteInsertQuery");
#endif
            #endregion
            if (ColumnValues.Count == 0)
            {
                foreach (var field in AllFields)
                    if (field.InternalName != PrimaryKeyField.InternalName && field.InternalName != "*")
                    {
                        ColumnValues.Add(field, null);
                        break;
                    }
                if (ColumnValues.Count == 0)
                    throw new dhtmlxException("Data row cannot be inserted because there were no data specified");
            }
            try
            {

                this.ExecuteNonQuery(this.CreateInsertQuery(TableName, ColumnValues));
                object result = this.ExecuteScalar(this.CreateLastInsertedQuery(TableName, PrimaryKeyField));
                #region LOG ENTRY
#if !NO_LOG
                Log.WriteLine(this, "Result: " + Convert.ToString(result));
#endif
                #endregion
                return result;
            }
            catch (SqlException ex)
            {
                #region LOG ENTRY
#if !NO_LOG
                Log.WriteLine(this, "Exception cought: " + ex.Message);
#endif
                #endregion
                throw new dhtmlxDBException("Insert operation failed with the following error: " + ex.Message);
            }
            catch (Exception ex)
            {
                #region LOG ENTRY
#if !NO_LOG
                Log.WriteLine(this, "Exception cought: " + ex.Message);
#endif
                #endregion
                throw new dhtmlxException("Insert operation failed with the following error: " + ex.Message);
            }
        }
        /// <summary>
        /// Creates T-SQL Select query based on parameters provided
        /// </summary>
        /// <param name="TableName">Table name to create query for</param>
        /// <param name="RequestedFields">Fields to include into result</param>
        /// <param name="Rules">Rules to apply to query</param>
        /// <param name="OrderBy">Order statements</param>
        /// <param name="StartIndex">Start index to take result rows from</param>
        /// <param name="Count">Number or rows to return</param>
        /// <returns>Ready-to-execute T-SQL query</returns>
        protected override string CreateSelectQuery(string TableName, IEnumerable<Field> RequestedFields, List<Rule> Rules, List<OrderByStatement> OrderBy, int StartIndex, int Count)
        {
            #region LOG ENTRY
#if !NO_LOG
            Log.WriteLine(this, string.Format("Creating SelectQuery from TableName: {0}, Fields: {1}, Rules: {2}, OrderBy: {3}, StartIndex: {4}, Count: {5}", TableName, Tools.Join(RequestedFields, ", "), Tools.Join(Rules, " AND "), Tools.Join(OrderBy, ", "), StartIndex, Count));
#endif
            #endregion
            StringBuilder Query = new StringBuilder();
            string result = "";
            if (RequestedFields == null && Rules == null && OrderBy == null && StartIndex == -1 && Count == -1)
            {
                Query.Append(TableName);
                result = Query.ToString();
            }
            else
            {
                Query.Append("SELECT ");
                Query.Append(Tools.Join(RequestedFields, ", "));
                Query.Append(" FROM ").Append(TableName);

                if (Rules != null)
                {
                    string WhereClause = Tools.Join(Rules, " AND ");
                    if (!string.IsNullOrEmpty(WhereClause))
                        Query.Append(" WHERE ").Append(WhereClause);

                }
                if (OrderBy != null)
                {
                    string OrderByStatements = Tools.Join(OrderBy, ", ");
                    if (!string.IsNullOrEmpty(OrderByStatements))
                        Query.Append(" ORDER BY ").Append(OrderByStatements);

                }
                if (Count > 0 && (Count != Int32.MaxValue || StartIndex > 0))
                {
                    Query.Append(string.Format(" LIMIT {0} OFFSET {1} ", Count.ToString(), (StartIndex).ToString()));
                }

                result = Query.ToString();

            }
            #region LOG ENTRY
#if !NO_LOG
            Log.WriteLine(this, "Select query: " + result);
#endif
            #endregion
            return result;
        }
    }
}
